<?xml version="1.0"?>

<!--
  * xmLPdoc, version alpha-2000/04/09.
  *
  * Pre-alpha XSLT example, showing how an (X)HTML document
  * with embedded code fragments can be woven using XSL
  * to generate literate documentation in HTML.
  *
  * The aim of this stylesheet is to mimic some of what
  * FunnelWeb 3.05AC could do in creating HTML documentation
  * (see "http://www.theoffice.net/abcoates/funnelweb.html").
  * However, this is really just a very early sample,
  * and subject to change without notice.
  *
  * Use as you will, no warranties of any kind, express or implied.
  * If you modify this and redistribute it, you must change the name
  * from "xmLP" to something sufficiently different that it will not
  * be confused with "xmLP".
  *
  * Name "xmLP" is copyright (C) 2000 Anthony B. Coates.
  -->

<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:lp="http://www.litprog.org/xmLP/alpha-2000-04-09/"
  xmlns="http://www.w3.org/TR/REC-html40"
  version="1.0"
>

<xsl:output method="html" indent="no"/>

  <!-- **** HTML tag support **** -->

  <!-- Any other tag:
       Tags not covered in other rules are copied through,
       on the assumption that they are HTML tags.
    -->
  <xsl:template match="*|@*">
    <xsl:copy>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <!-- Generate the HEAD, with a CSS link
    -->
  <xsl:template match="HEAD">
    <xsl:copy>
      <xsl:apply-templates/>
      <xsl:element name="LINK">
        <xsl:attribute name="TYPE">text/css</xsl:attribute>
        <xsl:attribute name="SRC">xmLPdoc.css</xsl:attribute>
        <xsl:attribute name="REL">stylesheet</xsl:attribute>
        <xsl:attribute name="HREF">xmLPdoc.css</xsl:attribute>
      </xsl:element>
    </xsl:copy>
  </xsl:template>

  <!-- **** xmLP Tags **** -->

  <!-- "lp:macro" tag:
       'xmLP' macro definition.
    -->
  <xsl:template match="lp:macro">
    <xsl:variable name="macroName" select="lp:name"/>
    <xsl:element name="DIV">
      <!-- Format DIV box differently depending on macro usage. -->
      <xsl:choose>
        <xsl:when test="@lp:usage='never'">
          <xsl:attribute name="CLASS">xmlp-z-macro-def</xsl:attribute>
        </xsl:when>
        <xsl:when test="@lp:usage='many'">
          <xsl:attribute name="CLASS">xmlp-macro-def</xsl:attribute>
        </xsl:when>
        <xsl:otherwise>
          <!-- "lp:usage='once'" is default. -->
          <xsl:attribute name="CLASS">xmlp-macro-def</xsl:attribute>
        </xsl:otherwise>
      </xsl:choose>
      <!-- Create an anchor for the macro. -->
      <xsl:element name="A">
        <xsl:attribute name="NAME">
          <xsl:text>xmLP[</xsl:text>
   	  <xsl:number level="single" count="lp:macro"/>
          <xsl:text>]PLmx</xsl:text>
        </xsl:attribute>
      </xsl:element>
      <!-- Use a display list as a way of doing indenting. -->
      <xsl:element name="DL">
        <xsl:element name="DT">
          <xsl:element name="EM">
            <!-- Add the macro name and a macro number. -->
            <xsl:apply-templates select="lp:name"/>
            <xsl:text> [</xsl:text>
   	    <xsl:number level="single" count="lp:macro"/>
            <xsl:text>] </xsl:text>
            <!-- Mark the macro if it can be used multiply. -->
            <xsl:if test="@lp:usage='many'">
              <xsl:text>M</xsl:text>
            </xsl:if>
            <!-- Use "+=" and not "=" if the macro can be added to. -->
            <xsl:choose>
              <xsl:when test="@lp:final='false'">
                <xsl:text>+=</xsl:text>
              </xsl:when>
              <xsl:otherwise>
                <!-- "lp:final='true'" is default. -->
                <xsl:text>=</xsl:text>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:element>
        </xsl:element>
        <!-- Insert the code for the macro. -->
        <xsl:element name="DD">
          <xsl:element name="PRE">
            <xsl:apply-templates select="lp:code"/>
          </xsl:element>
        </xsl:element>
      </xsl:element>
      <!-- Check how many times the same macro name was defined. -->
      <xsl:variable name="macroDefinitions"
        select="//lp:macro/lp:name[text()=$macroName]"/>
      <xsl:variable name="macroDefinitionCount" select="count($macroDefinitions)"/>
      <xsl:element name="H5">
        <xsl:choose>
          <xsl:when test="@lp:final='false'">
            <!-- Additive macro, so print out where it is defined. -->
            <xsl:text>This macro is defined in definitions</xsl:text>
            <xsl:for-each select="$macroDefinitions">
              <xsl:text> </xsl:text>
              <xsl:element name="A">
                <xsl:attribute name="HREF">
                  <xsl:text>#xmLP[</xsl:text>
                  <xsl:number count="lp:macro"/>
                  <xsl:text>]PLmx</xsl:text>
                </xsl:attribute>
     	      <xsl:number count="lp:macro"/>
              </xsl:element>
            </xsl:for-each>
            <xsl:text>.</xsl:text>
            <!-- Warn if an additive macro has only one section. -->
            <xsl:if test="$macroDefinitionCount &lt; 2">
              <xsl:variable name="warningText">This macro is additive, but does not have multiple definitions</xsl:variable>
              <xsl:element name="BR"></xsl:element>
              <xsl:element name="SPAN">
                <xsl:attribute name="CLASS">xmlp-warning</xsl:attribute>
                <xsl:text>Warning: </xsl:text>
                <xsl:value-of select="$warningText"/>
                <xsl:text>.</xsl:text>
              </xsl:element>
              <xsl:call-template name="xmlp-warning">
                <xsl:with-param name="text" select="$warningText"/>
                <xsl:with-param name="location"
                ><xsl:call-template name="macro-location"
                /></xsl:with-param>
              </xsl:call-template>
            </xsl:if>
          </xsl:when>
          <xsl:otherwise><!-- non-additive macro -->
            <!-- Error if a non-additive macro is multiply defined. -->
            <xsl:if test="$macroDefinitionCount != 1">
              <xsl:variable name="errorText">This macro is not additive.  It should have been defined once and only once</xsl:variable>
              <xsl:element name="BR"></xsl:element>
              <xsl:element name="SPAN">
                <xsl:attribute name="CLASS">xmlp-error</xsl:attribute>
                <xsl:text>Error: </xsl:text>
                <xsl:value-of select="$errorText"/>
                <xsl:text>.</xsl:text>
              </xsl:element>
              <xsl:call-template name="xmlp-error">
                <xsl:with-param name="text" select="$errorText"/>
                <xsl:with-param name="location"
                ><xsl:call-template name="macro-location"
                /></xsl:with-param>
              </xsl:call-template>
            </xsl:if>
          </xsl:otherwise>
        </xsl:choose>
        <!-- Check how many times the macro was invoked. -->
        <xsl:variable name="macroInvocations"
          select="//lp:macro/lp:code/lp:invoke/lp:name[text()=$macroName]"/>
        <xsl:variable name="macroInvocationCount"
          select="count($macroInvocations)"/>
        <xsl:variable name="fileInvocations"
          select="//lp:file/lp:code/lp:invoke/lp:name[text()=$macroName]"/>
        <xsl:variable name="fileInvocationCount"
          select="count($fileInvocations)"/>
        <xsl:variable name="totalInvocationCount"
          select="$macroInvocationCount + $fileInvocationCount"/>
        <xsl:variable name="invokedNever"
          select="$totalInvocationCount &lt;= 0"/>
        <xsl:variable name="invokedOnce"
          select="$totalInvocationCount = 1"/>
        <xsl:variable name="invokedMany"
          select="$totalInvocationCount &gt; 1"/>
        <xsl:if test="$macroInvocationCount &gt;= 1">
          <!-- Print out usages in other macros. -->
          <xsl:element name="BR"></xsl:element>
          <xsl:text>This macro is invoked in macros</xsl:text>
          <xsl:for-each select="$macroInvocations">
            <xsl:text> </xsl:text>
            <xsl:element name="A">
              <xsl:attribute name="HREF">
                <xsl:text>#xmLP[</xsl:text>
                <xsl:number count="lp:macro"/>
                <xsl:text>]PLmx</xsl:text>
              </xsl:attribute>
   	      <xsl:number count="lp:macro"/>
            </xsl:element>
          </xsl:for-each>
          <xsl:text>.</xsl:text>
        </xsl:if>
        <xsl:if test="$fileInvocationCount &gt;= 1">
          <!-- Print out direct usages in product files. -->
          <xsl:element name="BR"></xsl:element>
          <xsl:text>This macro is invoked in files</xsl:text>
          <xsl:for-each select="$fileInvocations">
            <xsl:text> </xsl:text>
            <xsl:element name="A">
              <xsl:attribute name="HREF">
                <xsl:text>#xmLPfile[</xsl:text>
                <xsl:number count="lp:file"/>
                <xsl:text>]elifPLmx</xsl:text>
              </xsl:attribute>
   	      <xsl:number count="lp:file"/>
            </xsl:element>
          </xsl:for-each>
          <xsl:text>.</xsl:text>
        </xsl:if>
        <!-- Note if macro is not actually used in tangling. -->
        <xsl:if test="$invokedNever">
          <xsl:element name="BR"></xsl:element>
          <xsl:text>This macro is NEVER invoked.</xsl:text>
        </xsl:if>
        <xsl:choose>
          <xsl:when test="@lp:usage='never'">
            <!-- Error if macro was used but was not intended for use. -->
            <xsl:if test="not($invokedNever)">
              <!-- Error if macro was used but was not intended for use. -->
              <xsl:variable name="errorText">This macro should not have been invoked</xsl:variable>
              <xsl:element name="BR"></xsl:element>
              <xsl:element name="SPAN">
                <xsl:attribute name="CLASS">xmlp-error</xsl:attribute>
                <xsl:text>Error: </xsl:text>
                <xsl:value-of select="$errorText"/>
                <xsl:text>.</xsl:text>
              </xsl:element>
              <xsl:call-template name="xmlp-error">
                <xsl:with-param name="text" select="$errorText"/>
                <xsl:with-param name="location"
                ><xsl:call-template name="macro-location"
                /></xsl:with-param>
              </xsl:call-template>
            </xsl:if>
          </xsl:when>
          <xsl:when test="@lp:usage='many'">
            <xsl:if test="not($invokedMany)">
            <!-- Warning if multiple use macro was used only once or
                 not at all. -->
              <xsl:variable name="warningText">This macro was expected to be invoked multiple times</xsl:variable>
              <xsl:element name="BR"></xsl:element>
              <xsl:element name="SPAN">
                <xsl:attribute name="CLASS">xmlp-warning</xsl:attribute>
                <xsl:text>Warning: </xsl:text>
                <xsl:value-of select="$warningText"/>
                <xsl:text>.</xsl:text>
              </xsl:element>
              <xsl:call-template name="xmlp-warning">
                <xsl:with-param name="text" select="$warningText"/>
                <xsl:with-param name="location"
                ><xsl:call-template name="macro-location"
                /></xsl:with-param>
              </xsl:call-template>
            </xsl:if>
          </xsl:when>
          <xsl:otherwise><!-- Usage is 'once' -->
            <xsl:if test="not($invokedOnce)">
              <!-- Error if macro was to be used once but wasn't. -->
              <xsl:variable name="errorText">This macro should have been invoked once and once only</xsl:variable>
              <xsl:element name="BR"></xsl:element>
              <xsl:element name="SPAN">
                <xsl:attribute name="CLASS">xmlp-error</xsl:attribute>
                <xsl:text>Error: </xsl:text>
                <xsl:value-of select="$errorText"/>
                <xsl:text>.</xsl:text>
              </xsl:element>
              <xsl:call-template name="xmlp-error">
                <xsl:with-param name="text" select="$errorText"/>
                <xsl:with-param name="location"
                ><xsl:call-template name="macro-location"
                /></xsl:with-param>
              </xsl:call-template>
            </xsl:if>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:element>
    </xsl:element>
  </xsl:template>  

  <!-- "lp:file" tag:
       'xmLP' product file definition.
    -->
  <xsl:template match="lp:file">
    <xsl:element name="DIV">
      <!-- Format DIV box differently for file usage. -->
      <xsl:attribute name="CLASS">xmlp-file-def</xsl:attribute>
      <!-- Create an anchor for the file definition. -->
      <xsl:element name="A">
        <xsl:attribute name="NAME">
          <xsl:text>xmLPfile[</xsl:text>
   	  <xsl:number level="single" count="lp:file"/>
          <xsl:text>]elifPLmx</xsl:text>
        </xsl:attribute>
      </xsl:element>
      <!-- Use a display list as a way of doing indenting. -->
      <xsl:element name="DL">
        <xsl:element name="DT">
          <xsl:element name="EM">
            <!-- Add the file name and a file number. -->
            <xsl:value-of select="@lp:filename"/>
            <xsl:text> [</xsl:text>
   	    <xsl:number level="single" count="lp:file"/>
            <xsl:text>] </xsl:text>
            <xsl:text>=</xsl:text>
          </xsl:element>
        </xsl:element>
        <!-- Insert the code for the file. -->
        <xsl:element name="DD">
          <xsl:element name="PRE">
            <xsl:apply-templates select="lp:code"/>
          </xsl:element>
        </xsl:element>
      </xsl:element>
      <!-- Note that this is a file. -->
      <xsl:element name="H5">
        <xsl:text>This is an output file.</xsl:text>
      </xsl:element>
    </xsl:element>
  </xsl:template>  

  <!-- "lp:code" tag:
       'xmLP' code macro.
    -->
  <xsl:template match="lp:code">
    <xsl:apply-templates select="lp:invoke|text()"/>
  </xsl:template>

  <!-- "lp:invoke" tag:
       'xmLP' macro invocation.
    -->
  <xsl:template match="lp:invoke">
    <I>
      <!-- Link the invocation back to the definition. -->
      <xsl:element name="A">
        <xsl:attribute name="HREF">
          <xsl:text>#xmLP[</xsl:text>
            <xsl:variable name="macroName" select="lp:name"/>
            <xsl:for-each select="//lp:macro/lp:name[text()=$macroName]">
              <xsl:if test="position()=1">
                <xsl:number count="lp:macro"/>
              </xsl:if>
            </xsl:for-each>
          <xsl:text>]LPmx</xsl:text>
        </xsl:attribute>
        <xsl:value-of select="lp:name"/>
      </xsl:element>
    </I>
  </xsl:template>

  <!-- "H1" tag.
       Adds a contents number to contents items.
    -->
  <xsl:template match="H1[@lp:contents='true']">
    <xsl:variable name="contentsItem"
    ><xsl:number count="/HTML/BODY/H1[@lp:contents='true']"
    /></xsl:variable>
    <H1>
      <xsl:element name="A">
        <xsl:attribute name="NAME">
          <xsl:value-of select="$contentsItem"/>
          <xsl:text> </xsl:text><xsl:value-of select="string(.)"/>
        </xsl:attribute>
        <xsl:value-of select="$contentsItem"/><xsl:text> </xsl:text>
        <xsl:apply-templates/>
      </xsl:element>
    </H1>
  </xsl:template>

  <!-- "lp:table-of-contents" tag.
       Builds a table of contents.
       Contents entries are H1 tags with an 'lp:contents="true"' attribute.
    -->
  <xsl:template match="lp:table-of-contents">
    <xsl:for-each select="/HTML/BODY/H1[@lp:contents='true']">
      <xsl:variable name="contentsItem"
      ><xsl:number count="/HTML/BODY/H1[@lp:contents='true']"
      /></xsl:variable>
      <H2>
        <xsl:element name="A">
          <xsl:attribute name="href">
            <xsl:text>#</xsl:text><xsl:value-of select="$contentsItem"/>
            <xsl:text> </xsl:text><xsl:value-of select="string(.)"/>
          </xsl:attribute>
          <xsl:value-of select="$contentsItem"/>
          <xsl:text> </xsl:text><xsl:value-of select="."/>
        </xsl:element>
      </H2>
    </xsl:for-each>
  </xsl:template>

  <!-- "lp:list-of-macros" tag.
       Builds a list of macros.
    -->
  <xsl:template match="lp:list-of-macros">
    <OL>
    <xsl:for-each select="//lp:macro">
      <xsl:variable name="macroItem"
      ><xsl:number count="//lp:macro"
      /></xsl:variable>
      <LI>
        <xsl:element name="A">
          <xsl:attribute name="href">
            <xsl:text>#xmLP[</xsl:text>
            <xsl:value-of select="$macroItem"/>
            <xsl:text>]PLmx</xsl:text>
          </xsl:attribute>
          <xsl:value-of select="lp:name"/>
        </xsl:element>
        <xsl:if test="@lp:final='false'">
          <xsl:text> [</xsl:text>
          <xsl:call-template name="additive-macro-index"/>
          <xsl:text>]</xsl:text>
        </xsl:if>
        <xsl:if test="@lp:usage='never'">
          <xsl:text> [never invoked]</xsl:text>
        </xsl:if>
      </LI>
    </xsl:for-each>
    </OL>
  </xsl:template>

  <!-- "lp:list-of-file" tag.
       Builds a list of files.
    -->
  <xsl:template match="lp:list-of-files">
    <OL>
    <xsl:for-each select="//lp:file">
      <xsl:variable name="fileItem"
      ><xsl:number count="//lp:file"
      /></xsl:variable>
      <LI>
        <xsl:element name="A">
          <xsl:attribute name="href">
            <xsl:text>#xmLPfile[</xsl:text>
            <xsl:value-of select="$fileItem"/>
            <xsl:text>]elifPLmx</xsl:text>
          </xsl:attribute>
          <xsl:value-of select="@lp:filename"/>
        </xsl:element>
      </LI>
    </xsl:for-each>
    </OL>
  </xsl:template>

  <!-- **** Other Tags **** -->

  <!-- "refer" tag:
       Adds a hyperlinked document reference.
    -->
  <xsl:template match="refer">
    <xsl:element name="A">
      <xsl:attribute name="HREF">
        <xsl:value-of select="@ref"/>
      </xsl:attribute>
      <xsl:if test="@target">
        <xsl:attribute name="TARGET">
          <xsl:value-of select="@target"/>
        </xsl:attribute>
      </xsl:if>
      <xsl:apply-templates/>
    </xsl:element>
  </xsl:template>

  <!-- "bookmark" tag:
       Adds a bookmark that can be referred to.
    -->
  <xsl:template match="bookmark">
    <xsl:element name="A">
      <xsl:attribute name="NAME">
        <xsl:value-of select="@name"/>
      </xsl:attribute>
      <xsl:apply-templates/>
    </xsl:element>
  </xsl:template>

<!-- Generates an xmLP message. -->
<xsl:template name="xmlp-message">
  <xsl:param name="status">error</xsl:param>
  <xsl:param name="text">No information available</xsl:param>
  <xsl:param name="location">unknown</xsl:param>
  <xsl:message>
    <xsl:text>xmLP </xsl:text>
    <xsl:value-of select="$status"/>
    <xsl:text>: [</xsl:text>
    <xsl:value-of select="$location"/>
    <xsl:text>]: </xsl:text>
    <xsl:value-of select="$text"/>
    <xsl:text>"</xsl:text>
  </xsl:message>
</xsl:template>

<!-- Generates an xmLP warning message. -->
<xsl:template name="xmlp-warning">
  <xsl:param name="text">No information available</xsl:param>
  <xsl:param name="location">unknown</xsl:param>
  <xsl:call-template name="xmlp-message">
    <xsl:with-param name="status">warning</xsl:with-param>
    <xsl:with-param name="text" select="$text"/>
    <xsl:with-param name="location" select="$location"/>
  </xsl:call-template>
</xsl:template>

<!-- Generates an xmLP error message. -->
<xsl:template name="xmlp-error">
  <xsl:param name="text">No information available</xsl:param>
  <xsl:param name="location">unknown</xsl:param>
  <xsl:call-template name="xmlp-message">
    <xsl:with-param name="status">error</xsl:with-param>
    <xsl:with-param name="text" select="$text"/>
    <xsl:with-param name="location" select="$location"/>
  </xsl:call-template>
</xsl:template>

<!-- Returns a location string for the current macro. -->
<xsl:template name="macro-location">
  <xsl:text>macro #</xsl:text>
  <xsl:number count="lp:macro"/>
  <xsl:text>: </xsl:text>
  <xsl:value-of select="lp:name"/>
</xsl:template>

<!-- Returns a location string for the current file. -->
<xsl:template name="file-location">
  <xsl:text>file #</xsl:text>
  <xsl:number count="lp:file"/>
  <xsl:text>: </xsl:text>
  <xsl:value-of select="@lp:filename"/>
</xsl:template>

<!-- Returns a index string for the current additive macro. -->
<xsl:template name="additive-macro-index">
  <xsl:variable name="currentPosition">
    <xsl:number count="lp:macro"/>
  </xsl:variable>
  <xsl:variable name="macroName" select="lp:name"/>
  <xsl:variable name="additiveMacros"
    select="//lp:macro/lp:name[text()=$macroName]"/>
  <xsl:text>#</xsl:text>
  <xsl:for-each select="$additiveMacros">
    <xsl:variable name="loopPosition">
      <xsl:number count="lp:macro"/>
    </xsl:variable>
    <xsl:if test="number($loopPosition) = number($currentPosition)">
      <xsl:value-of select="position()"/>
    </xsl:if>
  </xsl:for-each>
  <xsl:text>/</xsl:text>
  <xsl:value-of select="count($additiveMacros)"/>
</xsl:template>

</xsl:stylesheet>

<!-- End of xmLPweave.xsl -->
